#include "cone.h"

/****************************************/
/* Lighting model.                      */
/****************************************/
void LIGHT_MODEL(void) {
GLfloat ambient[]    = {0.01, 0.01, 0.01, 	1.0}; 	// Ambient
GLfloat diffuseD[]   = {0.0, 0.7, 1.0, 1.0};		// Diffuse
GLfloat PositionD[]  = {lightX, lightY, lightZ, 0.0};
GLfloat specularS[]  = {1.0, 0.0, 0.0, 1.0}; 		// Specular
GLfloat PositionS[]  = {1000.0, 0.0, 200.0, 0.0};
GLfloat direction[]  = {  -1.0, 0.3,  -0.3};
   glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR); 
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   // Ambient "light source".
   glColorMaterial(GL_FRONT, GL_AMBIENT);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   // Diffuse light source.
   glColorMaterial(GL_FRONT, GL_DIFFUSE);
   glLightfv(GL_LIGHT1, GL_DIFFUSE,  diffuseD);
   glLightfv(GL_LIGHT1, GL_POSITION, PositionD);
   // Specular light source.
   glColorMaterial(GL_FRONT, GL_SPECULAR);
   glLightfv(GL_LIGHT2, GL_DIFFUSE,  specularS);
   glLightfv(GL_LIGHT2, GL_SPECULAR, specularS);
   glLightfv(GL_LIGHT2, GL_POSITION, PositionS);
   glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, direction);
//   glLightf(GL_LIGHT1, GL_CONSTANT_ATTENUATION, 1.5);
//   glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.5);
//   glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.2);
   glLightf(GL_LIGHT2, GL_SPOT_CUTOFF, 25);
   glLightf(GL_LIGHT2, GL_SPOT_EXPONENT, 5.0);  
}

/****************************************/
/* Draws the cone in the CONE window.   */
/*      v1                              */
/*     /  \                             */
/*    /    \                            */
/*   /      \                           */
/*  v2------v3                          */
/****************************************/
void DRAW_CONE(void) {
int i;
int a, b;
GLfloat x1, x2, x3;
GLfloat y1, y2, y3;
GLfloat z1, z2, z3;
GLfloat mag;
GLfloat X[161], X1[161], X2[161], X3[161];
GLfloat Y[161], Y1[161], Y2[161], Y3[161];
GLfloat Z[161], Z1[161], Z2[161], Z3[161];
double  angleStep, angle1, angle2;
   // Adjust location and view angle.
   glLoadIdentity();
   glTranslated(400, 300, 0);
   glRotated(yAngle, 0, 1, 0);  // CW from above (YAW)
   glRotated(pAngle, 0, 0, 1);  // CW from viewer (PITCH)
   glRotated(rAngle, 1, 0, 0);  // CW from side (ROLL)
   // Build a cone.
   angleStep = (2.0*pi) / (double)(faces-1);
   angle1 = 0.0;
   for (i = 1; i < faces; i++) {
       angle2 += angleStep;
       // Define vertices.
       x1 = 0.0;						// vertix 1 (CCW)
       y1 = 0.0;
       z1 = scale;
       x2 = 0.5 * scale * cos(angle1);	// vertix 2
       y2 = 0.5 * scale * sin(angle1);
       z2 = 0.0;  			
       x3 = 0.5 * scale * cos(angle2);	// vertix 3
       y3 = 0.5 * scale * sin(angle2);
       z3 = 0.0;
       // Determine surface normal (UNIT) vector.
       X[i] = (y1-y2)*(z1+z2)+(y2-y3)*(z2+z3)+(y3-y1)*(z3+z1);
       Y[i] = (z1-z2)*(x1+x2)+(z2-z3)*(x2+x3)+(z3-z1)*(x3+x1);
       Z[i] = (x1-x2)*(y1+y2)+(x2-x3)*(y2+y3)+(x3-x1)*(y3+y1);
       // Record vertices.
       X1[i] = x1; X2[i] = x2; X3[i] = x3;
       Y1[i] = y1; Y2[i] = y2; Y3[i] = y3;
       Z1[i] = z1; Z2[i] = z2; Z3[i] = z3;
       // Increment rotation.
       angle1 = angle2;
   }
   glEnable(GL_NORMALIZE);
   for (i = 1; i < faces; i++) {
       // Adjust vertex normals to fit shading model.
       if (shader == 0) {		// Assign surface normal vectors for FLAT shading.
          x1 = X[i]; y1 = Y[i]; z1 = Z[i]; 
          x2 = X[i]; y2 = Y[i]; z2 = Z[i]; 
          x3 = X[i]; y3 = Y[i]; z3 = Z[i];
       }
	   if (shader == 1) {		// Assign averaged normal vectors for SMOOTH shading.
          a = i-1;
          b = i+1;
          if (i == 1) a = faces-1;
          if (b == faces) b = 1;
          x1 = X[a]+X[i]+X[b];	// Average this vertex with the one to the left and the right.
          y1 = Y[a]+Y[i]+Y[b];
          z1 = Z[a]+Z[i]+Z[b];
          x2 = X[a]+X[i];		// Average this vertex with the one to the left.
          y2 = Y[a]+Y[i];
          z2 = Z[a]+Z[i];
          x3 =      X[i]+X[b];	// Average this vertex with the one to the right.
          y3 =      Y[i]+Y[b];
          z3 =      Z[i]+Z[b];
       }
       // Draw
       glBegin(GL_TRIANGLES);
         glNormal3f(x1, y1, z1);
         glVertex3f(X1[i], Y1[i], Z1[i]);
         glNormal3f(x2, y2, z2);
         glVertex3f(X2[i], Y2[i], Z2[i]); 
         glNormal3f(x3, y3, z3);
         glVertex3f(X3[i], Y3[i], Z3[i]);
       glEnd();
   }
}

/****************************************/
/* Displays cone in the CONE window.    */
/****************************************/
void SCENE_DISPLAY(void) {
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   // Hidden surface removal.
   glEnable(GL_CULL_FACE); 
   glEnable(GL_DEPTH_TEST);
   // Lighting.
   glEnable(GL_LIGHTING);
   if (lightA == 1) glEnable(GL_LIGHT0);
               else glDisable(GL_LIGHT0);
   if (lightD == 1) glEnable(GL_LIGHT1);
               else glDisable(GL_LIGHT1);
   if (lightS == 1) glEnable(GL_LIGHT2);
               else glDisable(GL_LIGHT2);
   DRAW_CONE();
   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);
   glLoadIdentity();
   glutSwapBuffers();
}

/****************************************/
/* Animates cone in the CONE window.    */
/****************************************/
void SCENE_ANIMATE(void) {
   pAngle += angleStep;
   system("sleep 0.1");
   SCENE_WINDOW.CLEAR(); 
   SCENE_DISPLAY();
}

/****************************************/
/* Main window display.                 */
/****************************************/
void MAIN_WINDOW(void) {
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glColor3f(0.6, 0.6, 0.6);
   glBegin(GL_QUADS);
     glVertex2i(       0,         0);
     glVertex2i(winWidth,         0);
     glVertex2i(winWidth, winHeight);
     glVertex2i(       0, winHeight);
   glEnd();
   LITA_BUTTON.DRAW();
   LITS_BUTTON.DRAW();
   LITD_BUTTON.DRAW();
   LTDU_BUTTON.DRAW();
   LTDC_BUTTON.DRAW();
   LTDD_BUTTON.DRAW();
   LTDL_BUTTON.DRAW();
   LTDR_BUTTON.DRAW();
   FACE_BUTTON.DRAW();
   SIZE_BUTTON.DRAW();
   ROLL_BUTTON.DRAW();
   SHAD_BUTTON.DRAW();
   EXIT_BUTTON.DRAW_HILITE(0.7, 0.0, 0.0);
   SCENE_WINDOW.DRAW();
   SCENE_WINDOW.CLEAR();
   glutSwapBuffers();
}

/************************************************************/
/* CALLBACK FUNTIONS.                                       */
/************************************************************/
/****************************************/
/* Mouse control.                       */
/****************************************/
void myMouse(int button, int state, int x, int y) {
   y = (winHeight - y);
   if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
      glFinish();
      // Ambient light.
      if (LITA_BUTTON.SELECTED(x, y)) { 
         LITA_BUTTON.CLICK();
         if (lightA == 0) {
            lightA = 1;
            LIGHT_MODEL(); 
            LITA_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         } else {
            lightA = 0;
            LITA_BUTTON.DRAW();
         }               
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      // Specular light.
      if (LITS_BUTTON.SELECTED(x, y)) { 
         LITS_BUTTON.CLICK();
         if (lightS == 0) {
            lightS = 1;
            LIGHT_MODEL(); 
            LITS_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         } else {
            lightS = 0;
            LITS_BUTTON.DRAW();
         }       
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      // Diffuse light.
      if (LITD_BUTTON.SELECTED(x, y)) { 
         LITD_BUTTON.CLICK();
         if (lightD == 0) {
            lightD = 1;
            lightX = 400.0;
            lightY = 1000.0;
            lightZ = 200.0;
            LIGHT_MODEL(); 
            LTDU_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
            LITD_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         } else {
            lightD = 0;
            LITD_BUTTON.DRAW();
            LTDL_BUTTON.DRAW();
            LTDU_BUTTON.DRAW();
            LTDC_BUTTON.DRAW();
            LTDD_BUTTON.DRAW();
            LTDR_BUTTON.DRAW();
         }         
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (LTDU_BUTTON.SELECTED(x, y)) { 
         if (lightD == 0) return;
         LTDU_BUTTON.CLICK();
         lightX = 400.0;
         lightY = 1000.0;
         lightZ = 200.0;
         LIGHT_MODEL(); 
         LTDL_BUTTON.DRAW();
         LTDU_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         LTDC_BUTTON.DRAW();
         LTDD_BUTTON.DRAW();
         LTDR_BUTTON.DRAW();
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (LTDC_BUTTON.SELECTED(x, y)) { 
         if (lightD == 0) return;
         LTDC_BUTTON.CLICK();
         lightX = 400.0;
         lightY = 350.0;
         lightZ = 5000.0;
         LIGHT_MODEL(); 
         LTDL_BUTTON.DRAW();
         LTDU_BUTTON.DRAW();
         LTDC_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         LTDD_BUTTON.DRAW();
         LTDR_BUTTON.DRAW();
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (LTDD_BUTTON.SELECTED(x, y)) { 
         if (lightD == 0) return;
         LTDD_BUTTON.CLICK();
         lightX = 400.0;
         lightY = -1000.0;
         lightZ = 200.0;
         LIGHT_MODEL(); 
         LTDD_BUTTON.DRAW();
         LTDU_BUTTON.DRAW();
         LTDC_BUTTON.DRAW();
         LTDD_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         LTDR_BUTTON.DRAW();
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (LTDL_BUTTON.SELECTED(x, y)) { 
         if (lightD == 0) return;
         LTDL_BUTTON.CLICK();
         lightX = -1000.0;
         lightY = 350.0;
         lightZ = 200.0;
         LIGHT_MODEL(); 
         LTDL_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         LTDU_BUTTON.DRAW();
         LTDC_BUTTON.DRAW();
         LTDD_BUTTON.DRAW();
         LTDR_BUTTON.DRAW();
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (LTDR_BUTTON.SELECTED(x, y)) { 
         if (lightD == 0) return;
         LTDR_BUTTON.CLICK();
         lightX = 1000.0;
         lightY = 350.0;
         lightZ = 200.0;
         LIGHT_MODEL(); 
         LTDL_BUTTON.DRAW();
         LTDU_BUTTON.DRAW();
         LTDC_BUTTON.DRAW();
         LTDD_BUTTON.DRAW();
         LTDR_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }

      if (FACE_BUTTON.SELECTED(x, y)) { 
         FACE_BUTTON.CLICK();
         faces *= 2;
         if (faces > 160) faces = 10; 
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }
      if (SIZE_BUTTON.SELECTED(x, y)) { 
         SIZE_BUTTON.CLICK();
         scale += 50;
         if (scale > 500) scale = 100; 
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }

      if (ROLL_BUTTON.SELECTED(x, y)) { 
         ROLL_BUTTON.CLICK();
         if (idle == 0) {
            glutIdleFunc(SCENE_ANIMATE);
            idle = 1;
            ROLL_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         } else {
            glutIdleFunc(NULL);
            idle = 0;
            ROLL_BUTTON.DRAW();
         }
      }
      if (SHAD_BUTTON.SELECTED(x, y)) { 
         SHAD_BUTTON.CLICK();
         if (shader == 0) {
            shader = 1;
            SHAD_BUTTON.DRAW_HILITE(0.7, 0.7, 0.0);
         } else {
            shader = 0;
            SHAD_BUTTON.DRAW();
         }         
         SCENE_WINDOW.CLEAR(); 
         SCENE_DISPLAY();
      }

      if (EXIT_BUTTON.SELECTED(x, y)) { 
         EXIT_BUTTON.CLICK();
         glutDestroyWindow(window);
         exit(0);                   
      }
   }
}

/****************************************/
/* Resizes screen (if required)         */
/****************************************/
void myResize(int width, int height) {
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)winWidth, 0.0, (GLdouble)winHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity(); 
}

/****************************************/
/* Displays screen                      */
/****************************************/
void myDisplay(void) {
   MAIN_WINDOW();
   SCENE_DISPLAY();
}

/****************************************/
/* Initializes OPENGL parameters    	*/
/****************************************/
void myInit(void) {
   glPointSize(1);
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
   glClear(GL_COLOR_BUFFER_BIT); 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)winWidth, 0.0, (GLdouble)winHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity(); 
}

/************************************************************/
/* MAIN.                                                    */
/************************************************************/
int main(int argc, char **argv) {
   glutInit(&argc, argv) ;
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);  
   glutInitWindowSize(winWidth, winHeight);
   glutInitWindowPosition(winX, winY);
   window = glutCreateWindow("CONE by R. Marsh.");
   glutDisplayFunc(myDisplay);
   glutReshapeFunc(myResize);
   glutMouseFunc(myMouse);
   myInit();
   glutMainLoop();
   return 1;
}
